package com.ruoyi.service.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.domain.TestRecords;
import com.ruoyi.domain.vo.CaseResultVO;
import com.ruoyi.domain.vo.StepResultVO;
import com.ruoyi.mapper.TestRecordsMapper;
import com.ruoyi.service.TestRecordsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;


@Transactional
@Service
public class TestRecordsServiceImpl extends ServiceImpl<TestRecordsMapper, TestRecords> implements TestRecordsService {

    @Autowired
    private TestRecordsService testRecordsService;

    @Autowired
    private TestRecordsMapper testRecordsMapper;


    @Override
    public void sendPost(String id,Long taskId) {

        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        map.put("verbose", "ALL");

        //以post形式请求接口
        String result= HttpUtil.post("https://10.12.48.78:8090/DescribeProjectTestResult",JSONObject.toJSONString(map));

        JSONObject jsonObject = JSONObject.parseObject(result);

        // 获取项目id
        String projectId = (String) jsonObject.get("id");

        // 获取项目用例结果列表
        List<CaseResultVO> caseResultVOS = jsonObject.getList("case_result_list", CaseResultVO.class);

        // 获取本地存储的列表
        List<TestRecords> recordsList = testRecordsService.findByTaskId(id);

        List<TestRecords> list = new ArrayList<>();
        if (caseResultVOS != null && caseResultVOS.size() != 0) {
            for (CaseResultVO caseResultVO : caseResultVOS) {
                // 检查 usecaseId 是否在 recordsList 中存在
                boolean exists = false;

                if (recordsList != null && recordsList.size() != 0) {

                    for (TestRecords records : recordsList) {
                        if (Objects.equals(caseResultVO.getId(), records.getUsecaseId())) {
                            exists = true;

                            if (!Objects.equals(caseResultVO.getStatus(), records.getTestResult())) {
                                records.setTestResult(caseResultVO.getStatus());
                                records.setRemediation(caseResultVO.getRemediation());
                                testRecordsService.updateById(records);
                            }
                            break;
                        }
                    }
                }

                if ((Objects.equals(caseResultVO.getStatus(), "PASSED") || Objects.equals(caseResultVO.getStatus(), "FAILED")) && !exists) {
                    TestRecords testRecords = new TestRecords();
                    testRecords.setProjectId(projectId);
                    testRecords.setTaskId(taskId);
                    testRecords.setUsecase(caseResultVO.getName());
                    testRecords.setUsecaseId(caseResultVO.getId());
                    testRecords.setDescription(caseResultVO.getDescription());
                    testRecords.setRiskLevel(caseResultVO.getRisk_level());
                    testRecords.setTestResult(caseResultVO.getStatus());
                    testRecords.setRemediation(caseResultVO.getRemediation());

                    // 用例步骤
                    List<StepResultVO> stepResultVOS = caseResultVO.getStep_result_list();

                    if (stepResultVOS != null && stepResultVOS.size() != 0) {
                        List<String> stepList = new ArrayList<>();
                        List<String> testDetailsList = new ArrayList<>();
                        int index = 1; // 序号从1开始
                        for (StepResultVO stepResultVO :stepResultVOS) {
                            // 测试方法
                            String stepName = stepResultVO.getName();

                            // 给每个元素加上序号
                            String stepWithIndex = index + "." + stepName;

                            stepList.add(stepWithIndex);
                            index++;

                            // 测试详情
                            String testDetails = stepResultVO.getDescription();
                            testDetailsList.add(testDetails);
                        }
                        testRecords.setTestMethod(StringUtils.join(stepList,"`"));
                        testRecords.setTestDetails(StringUtils.join(testDetailsList,"\n"));
                    }

                    list.add(testRecords);

                }

            }

            testRecordsService.saveBatch(list);
        }
    }

    @Override
    public void singleSendPost(String id, Long taskId) {
        Map<String, Object> map = new HashMap<>();
        map.put("id", id);
        map.put("verbose", "ALL");

        //以post形式请求接口
        String result= HttpUtil.post("https://10.12.48.78:8090/DescribeProjectTestResult",JSONObject.toJSONString(map));

        JSONObject jsonObject = JSONObject.parseObject(result);

        // 获取项目id
        String projectId = (String) jsonObject.get("id");

        // 获取项目用例结果列表
        List<CaseResultVO> caseResultVOS = jsonObject.getList("case_result_list", CaseResultVO.class);

        // 获取当前任务本地存储的列表
        List<TestRecords> recordsList = testRecordsService.findByTaskId(id);
        if (recordsList != null && recordsList.size() != 0) {
            testRecordsService.removeBatchByIds(recordsList);
        }

        List<TestRecords> list = new ArrayList<>();
        if (caseResultVOS != null && caseResultVOS.size() != 0) {
            for (CaseResultVO caseResultVO : caseResultVOS) {

                if (Objects.equals(caseResultVO.getStatus(), "PASSED") || Objects.equals(caseResultVO.getStatus(), "FAILED")) {
                    TestRecords testRecords = new TestRecords();
                    testRecords.setProjectId(projectId);
                    testRecords.setTaskId(taskId);
                    testRecords.setUsecase(caseResultVO.getName());
                    testRecords.setUsecaseId(caseResultVO.getId());
                    testRecords.setDescription(caseResultVO.getDescription());
                    testRecords.setRiskLevel(caseResultVO.getRisk_level());
                    testRecords.setTestResult(caseResultVO.getStatus());
                    testRecords.setRemediation(caseResultVO.getRemediation());

                    // 用例步骤
                    List<StepResultVO> stepResultVOS = caseResultVO.getStep_result_list();

                    if (stepResultVOS != null && stepResultVOS.size() != 0) {
                        List<String> stepList = new ArrayList<>();
                        List<String> testDetailsList = new ArrayList<>();
                        int index = 1; // 序号从1开始
                        for (StepResultVO stepResultVO :stepResultVOS) {
                            // 测试方法
                            String stepName = stepResultVO.getName();

                            // 给每个元素加上序号
                            String stepWithIndex = index + "." + stepName;

                            stepList.add(stepWithIndex);
                            index++;

                            // 测试详情
                            String testDetails = stepResultVO.getDescription();
                            testDetailsList.add(testDetails);
                        }
                        testRecords.setTestMethod(StringUtils.join(stepList,"`"));
                        testRecords.setTestDetails(StringUtils.join(testDetailsList,"\n"));
                    }

                    list.add(testRecords);

                }

            }

            testRecordsService.saveBatch(list);
        }
    }

    @Override
    public List<TestRecords> findByTaskId(String id) {
        return testRecordsMapper.findByTaskId(id);
    }
}


