package com.ruoyi.service.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.domain.TestUseCase;
import com.ruoyi.domain.vo.CaseResultVO;
import com.ruoyi.domain.vo.TestUsecaseVO;
import com.ruoyi.service.TestUseCaseService;
import com.ruoyi.mapper.TestUseCaseMapper;
import com.ruoyi.web.request.TestUseCaseByScenarioRequest;
import com.ruoyi.web.request.TestUseCaseIdListRequest;
import com.ruoyi.web.request.TestUserCaseRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* @author wangfei
* @description 针对表【t_test_usecase(测试用例)】的数据库操作Service实现
* @createDate 2024-02-18 13:41:28
*/
@Service
@Transactional
public class TestUseCaseServiceImpl extends ServiceImpl<TestUseCaseMapper, TestUseCase> implements TestUseCaseService{

    private String describeCaseList = "https://10.12.48.78:8090/DescribeCaseList";

    @Autowired
    private TestUseCaseMapper testUseCaseMapper;

    /**
     * 查询测试用例列表
     *
     * @param testUserCaseRequest 测试用例库
     * @return 测试用例集合
     */
    @Override
    public List<TestUseCase> selectseCaseList(TestUserCaseRequest testUserCaseRequest){
        if(testUserCaseRequest.getSearchKeywords() != null && (testUserCaseRequest.getSearchKeywords().contains("%") ||testUserCaseRequest.getSearchKeywords().contains("_")  )){
            testUserCaseRequest.setSearchKeywords(testUserCaseRequest.getSearchKeywords().toString().replaceAll("%","/%").replaceAll("_","/_"));
        }
        return testUseCaseMapper.selectseCaseList(testUserCaseRequest);
    }

    @Override
    public List<String> selectCaseIdList(TestUseCaseIdListRequest request) {
        List<TestUsecaseVO> list = testUseCaseMapper.selectListByTaskId(request.getModelTestId());
        return list.stream().map(TestUsecaseVO::getCaseId).collect(Collectors.toList());
    }

    /**
     * 通过测试场景获取所绑定的测试用例
     * @param request
     * @return
     */
    @Override
    public List<String> selectCaseByScenario(TestUseCaseByScenarioRequest request) {
        // 根据场景名遍历查询所绑定用例
        List<String> resultList = new ArrayList<>();
        request.getScenarioNameList()
                .forEach(scenarioName -> {
                    Map<String, Object> map = new HashMap<>();
                    map.put("scenario", scenarioName);
                    JSONObject jsonObject = callThirdPartyInterface(describeCaseList, map);
                    List<String> list = jsonObject.getList("case_list", CaseResultVO.class)
                            .stream().map(CaseResultVO::getId).collect(Collectors.toList());
                    resultList.addAll(list);
                });
        return resultList;
    }

    /**
     * 调用第三方接口
     * @param url
     * @param map
     * @return
     */
    @Override
    public JSONObject callThirdPartyInterface(String url, Map<String, Object> map) {
        String result = HttpUtil.post(url, JSONObject.toJSONString(map));
        return JSONObject.parseObject(result);
    }

}




