package com.ruoyi.web;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.service.ModelTestTaskService;
import com.ruoyi.web.request.CarReviewTaskViewRequest;
import com.ruoyi.web.request.ModelTestTaskConfirmRequest;
import com.ruoyi.web.request.ModelTestTaskSubmitRequest;
import com.ruoyi.web.response.ModelTestResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 车型试验任务Controller
 * 总任务包含 (后)-> NEW (可以)-> 查看、开始
 * 开始任务 -> PENDING -> 确认
 * 确认任务 -> SIGNED -> 查看、提交
 * 提交任务 -> FINISH -> 查看
 * @author gxk
 */
@Api(tags = "车型试验子任务")
@RestController
@RequestMapping("/car/test/task")
public class ModelTestTaskController extends BaseController {

    @Autowired
    private ModelTestTaskService modelTestTaskService;

    /**
     * 查看车型试验任务详情
     * @param request
     * @return
     */
    @ApiOperation("查看车型试验任务详情")
    @PostMapping("/getById")
    public R<ModelTestResponse> view(@Validated @RequestBody CarReviewTaskViewRequest request) {
        ModelTestResponse modelTestTaskList = modelTestTaskService.viewModelTestTask(request.getTaskId());
        return R.ok(modelTestTaskList);
    }

    /**
     * 开始车型试验任务
     * @param request
     * @return
     */
    @ApiOperation("开始车型试验任务")
    @PostMapping("/start")
    public R<String> startModel(@Validated @RequestBody CarReviewTaskViewRequest request) {
        modelTestTaskService.startModelTestTask(request.getTaskId());
        return R.ok();
    }

    /**
     * 确认车型试验任务
     * @param request
     * @return
     */
    @ApiOperation("确认车型试验任务")
    @PostMapping("/confirm")
    public R<String> confirmModel(@Validated @RequestBody ModelTestTaskConfirmRequest request) {
        modelTestTaskService.confirmModelTestTask(request.getTaskId());
        return R.ok();
    }

    /**
     * 提交车型试验任务 - 在待签字页面进行
     * @param request
     * @return
     */
    @ApiOperation("提交车型试验任务")
    @PostMapping("/submit")
    public R<String> submitModel(@Validated @RequestBody ModelTestTaskSubmitRequest request) {
        modelTestTaskService.submitModelTestTask(request);
        return R.ok();
    }

}
